class PackageWorker
  include Sidekiq::Worker
  require 'zip'

  sidekiq_options queue: :package_cache

  def perform(objects_ids = nil, filename = nil, cache_key = nil)
    return false if objects_ids.blank? || filename.blank? || cache_key.blank?
    @cache_key = cache_key

    return true if file_exist?(PackageService.file_root(filename))

    Rails.cache.write(PackageService.job_key(cache_key), 'wait')

    files = open_files(objects_ids)
    create_package(filename, files)
  ensure
    close_files(files) unless files.nil?
    job_key = PackageService.job_key(cache_key)
    Rails.cache.delete(job_key) if Rails.cache.exist?(job_key)
  end

  private

  def file_exist?(path)
    if File.exist?(path)
      cache_fetch(path.split('/').last)
      return true
    end
    false
  end

  def open_files(objects_ids)
    objects = objects_ids.map { |o| o['class'].constantize.find(o['id']) }

    files = []
    objects.each do |object|
      if object.is_a? Collection
        object.learning_objects.each { |o| files.concat object_files(o) }
      else
        files.concat object_files(object)
      end
    end

    files
  end

  def object_files(object)
    files = []
    object.attachments.where(bundle_name: 'ORIGINAL').each do |a|
      link = a.retrieve_cache_link
      link = a.id_dspace.to_s if link =~ %r{^https?:\/\/.*}i
      file = open_file(link)
      files << file unless file.nil?
    end
    files
  end

  def create_package(filename, files)
    FileUtils.mkdir_p(PackageService.file_root)
    Zip::File.open(PackageService.file_root(filename), Zip::File::CREATE) do |zipfile|
      files.each { |file| zipfile.add(File.basename(file.path), file.path) }
    end
    cache_fetch(filename)
  rescue => e
    file = PackageService.file_root(filename)
    FileUtils.rm(file) if File.exist?(file)
    raise e
  end

  def open_file(file_path)
    path = Rails.root.join('public', file_path)
    return File.open(path) if File.exist? path
    open_dspace_file(file_path)
  end

  def open_dspace_file(id)
    dspace_client = DspaceService.create_client
    file = dspace_client.bitstreams.retrieve(id: id)
    unless file.nil?
      file.open
      FileUtils.chmod 0664, file.path
    end
    file
  end

  def close_files(files)
    files.each do |file|
      path = file.path.to_s
      file.close
      FileUtils.rm(path) if path =~ %r{^\/tmp\/.*} && File.exist?(path)
    end
  end

  def cache_fetch(value)
    filename = Rails.cache.fetch(@cache_key)
    file = PackageService.file_root(filename) unless filename.nil?
    FileUtils.rm(file) if !file.nil? && File.exist?(file)

    Rails.cache.delete(@cache_key)
    Rails.cache.write(@cache_key, value)
  end
end
